/*--------------------------------------------------------------------*\
  File:      FontMenu.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the FontMenu class.
  ID:        $Id: FontMenu.h,v 1.22 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <Menu.h>


#ifndef FONT_MENU_H
#define FONT_MENU_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifndef DECL_SPEC

#if defined DLL_EXPORT_SYMBOLS
#define DECL_SPEC __declspec(dllexport)
#elif defined DLL_IMPORT_SYMBOLS
#define DECL_SPEC __declspec(dllimport)
#else
#define DECL_SPEC
#endif

#endif    // DECL_SPEC


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifdef DLL_EXPORT_SYMBOLS
#pragma export on
#endif

/*--------------------------------------------------------------------*\
  Class name:       FontMenu
  Inherits from:    public BMenu
  New data members: public static const uint32 mk_msg_type - the type
                        of BMessage this item will send to the BWindow
                        to which it is attached.
                    public static const char * const mk_msg_fam - the
                        name of the font family in the BMessage.
                    public static const char * const mk_msg_stl - the
                        name of the font style in the BMessage.
                    private font_family m_family - the menu's current
                        font family.
                    private font_style m_style - the menu's current
                        font style.
  Description: Class to implement a font menu in which each font's
      family is displayed as a submenu of the font family's styles.
      The submenu's titles are displayed  in case-insensitive
      alphabetical order.
      
      The FontMenu class is descended directly from the BMenu class,
      and supports all constructors of the BMenu class with one minor
      alteration. Note that there is a default argument for the first
      argument to the first constructor. As a matter of convenience
      this merely serves to set the title of the FontMenu to "Font",
      so constructing it can be as easy as calling the default
      constructor.
      
      Once instantiated, the FontMenu is empty. It contains no font
      families or styles. It must be initalized before use.
      The Initialize() member function adds the font families as
      submenus containing the families' styles in case-insensitive
      alphabetical order:
      
        FontMenu m_font_menu(new FontMenu());
        m_font_menu->Initialize();
      
      Once initialized and added to a window, the FontMenu becomes
      active and may start sending messages of type
      FontMenu::mk_msg_type. The messages contain two string fields.
      The first string is the font's family name and has the field
      name of FontMenu::mk_msg_fam. The second string is the font's
      style name and has the field name of FontMenu::mk_msg_stl. The
      messages are sent whenever a family menu item or style submenu
      item is selected. A BWindow need only respond to them in their
      MessageReceived() function:
      
        void MyWindow::MessageReceived(BMessage *a_msg)
        {
          if (a_msg->what == FontMenu::mk_msg_type)
          {
            m_font_menu->SetFamilyAndStyle(a_msg);
            // ...
          }
        }
      
      If the FontMenu is attatched to a BWindow, then it will
      automagically get deleted when the window is deleted.
\*--------------------------------------------------------------------*/

class DECL_SPEC FontMenu : public BMenu
{
	public:
	
		// Public static const data members
		
		static const uint32 mk_msg_type;
		static const char * const mk_msg_fam;
		static const char * const mk_msg_stl;
		
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: FontMenu
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const char * const a_title - the title of the menu.
                     Default: "Font".
                 menu_layout a_layout - the layout of the menu.
                     Default: B_ITEMS_IN_COLUMN.
  Throws:        none
  Description: Default class constructor.
\*--------------------------------------------------------------------*/
		
		FontMenu(const char * const a_title = "Font", menu_layout a_layout = B_ITEMS_IN_COLUMN);
		
/*--------------------------------------------------------------------*\
  Function name: FontMenu
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const char * const a_title - the title of the menu.
                 float a_width - the width of the menu.
                 float a_height - the height of the menu.
  Throws:        none
  Description: Class constructor.
\*--------------------------------------------------------------------*/
		
		FontMenu(const char * const a_title, float a_width, float a_height);
		
/*--------------------------------------------------------------------*\
  Function name: FontMenu
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     BMessage *a_archive - the archive from which to
                     reconstruct the menu.
  Throws:        none
  Description: Class constructor. Note: after calling this
      constructor, one must still call Initialize to add the fonts to
      the menu.
\*--------------------------------------------------------------------*/
		
		FontMenu(BMessage *a_archive);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~FontMenu
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~FontMenu(void);
		
/*--------------------------------------------------------------------*\
  Function name: FamilyAndStyle const
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     font_family &a_family - an array to store a copy of
                     the menu's font family.
                 font_style &a_style - an array to store a copy of the
                     menu's font style.
  Returns:       none
  Throws:        none
  Description: Function to get copies of the menu's font family and
      style.
\*--------------------------------------------------------------------*/
		
		void FamilyAndStyle(font_family &a_family, font_style &a_style) const;
		
/*--------------------------------------------------------------------*\
  Function name: virtual Initialize
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const bool a_render_style - a flag to indicate
                     whether or not the font's style should be
                     rendered in the font which it represents.
                     Default: true.
                 const bool a_render_family - a flag to indicate
                     whether or not the font's family should be
                     rendered in the font which it represents.
                     Default: false.
  Returns:       none
  Throws:        none
  Description: Function to initialize the font menu (to read all the
      families and styles, alphabetize them and stick them in the
      menu).
\*--------------------------------------------------------------------*/
		
		virtual void Initialize(const bool a_render_style = true, const bool a_render_family = false);
		
/*--------------------------------------------------------------------*\
  Function name: virtual MessageReceived
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     BMessage *a_msg - the message sent from a menu item.
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void MessageReceived(BMessage *a_msg);
		
/*--------------------------------------------------------------------*\
  Function name: SetFamilyAndStyle
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const BMessage *a_msg - the message sent from a menu item.
  Returns:       none
  Throws:        none
  Description: Function to set the current font family and style of
      the menu.
\*--------------------------------------------------------------------*/
		
		void SetFamilyAndStyle(const BMessage *a_msg);
		
/*--------------------------------------------------------------------*\
  Function name: SetFamilyAndStyle
  Member of:     public FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const font_family &a_family - the new font family.
                     Default: NULL.
                 const font_style &a_style - the new font style.
                     Default: NULL.
  Returns:       none
  Throws:        none
  Description: Function to set the current font family and style of
      the menu. If a_family is null, then the currently selected font
      family is deselected in the menu. Likewise if a_style is null,
      then the currently selected style is deselected.
\*--------------------------------------------------------------------*/
		
		void SetFamilyAndStyle(const font_family &a_family = NULL, const font_style &a_style = NULL);
	
	protected:
	
		// Protected member functions
		
/*--------------------------------------------------------------------*\
  Function name: FontMenu
  Member of:     protected FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     BRect a_frame - the menu's view's frame.
                 const char * const a_view_name - the menu's view's
                     name.
                 uint32 a_resize_mask - the menu's resize mask.
                 uint32 a_flags - the menu's flags.
                 menu_layout a_layout - the menu's layout.
                 bool a_resize_to_fit - true if the menu should resize
                     to fit, false otherwise.
  Throws:        none
  Description: Class constructor. This is to maintain compatibility
      with the BMenu class and any classes which may be derived from
      FontMenu.
\*--------------------------------------------------------------------*/
		
		FontMenu(BRect a_frame, const char * const a_view_name, uint32 a_resize_mask, uint32 a_flags, menu_layout a_layout, bool a_resize_to_fit);
	
	private:
	
		typedef BMenu Inherited;
		
		// Private static data members
		font_family m_family;
		font_style m_style;
		
		// Private static member functions
		
/*--------------------------------------------------------------------*\
  Function name: static MenuItemComp
  Member of:     private FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     BMenuItem * const * const a_1 - the first BMenuItem
                     to compare.
                 BMenuItem * const * const a_2 - the second BMenuItem
                     to compare.
  Returns:       int - returns a la strcmp().
  Throws:        none
  Description: Hook function used to sort the BMenuItem's by name.
\*--------------------------------------------------------------------*/
		
		static int MenuItemComp(BMenuItem * const * const a_1, BMenuItem * const * const a_2);
		
		// Private member functions
		
/*--------------------------------------------------------------------*\
  Function name: ConstructFamilyMenuItem
  Member of:     private FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     const font_family &a_family - the font family from
                     which to construct the submenu item.
                 const bool a_render_style - a flag to indicate
                     whether or not the font's style should be
                     rendered in the font which it represents.
                     Default: true.
                 const bool a_render_family - a flag to indicate
                     whether or not the font's family should be
                     rendered in the font which it represents.
                     Default: false.
  Returns:       BMenuItem * - the menu constructed on success, null
                     on failure (probably due to a memory allocation
                     error).
  Throws:        none
  Description: Function to construct a submenu item from a given font
      family.
\*--------------------------------------------------------------------*/
		
		BMenuItem *ConstructFamilyMenuItem(const font_family &a_family, const bool a_render_style = true, const bool a_render_family = false);
		
/*--------------------------------------------------------------------*\
  Function name: Init
  Member of:     private FontMenu
  Defined in:    FontMenu.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Function to initalize the menu.
\*--------------------------------------------------------------------*/
		
		void Init(void);
};

#ifdef DLL_EXPORT_SYMBOLS
#pragma export off
#endif


#endif    // FONT_MENU_H
