/*--------------------------------------------------------------------*\
  File:      PMain.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      PMain class.
  ID:        $Id: PMain.cpp,v 1.14 1998/07/01 05:16:16 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "PMain.h"
#include <limits.h>
#include <signal.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the PMain public static const data members
const char PMain::mk_path_delim('/');
const int PMain::mk_exit_code_no_err(0);
const int PMain::mk_exit_code_err(PMain::mk_exit_code_no_err - 1);
const int PMain::mk_exit_code_bad_args(PMain::mk_exit_code_err - 1);
const int PMain::mk_exit_code_no_memory(PMain::mk_exit_code_bad_args - 1);
const int PMain::mk_exit_code_bad_file(PMain::mk_exit_code_no_memory - 1);
const int PMain::mk_exit_code_last(PMain::mk_exit_code_bad_file);

// Initialize the PMain protected static const data members
const int PMain::mk_first_arg_ind(1);
const char PMain::mk_opt_esc('-');

// Initialize the PMain private static data members
PMain *PMain::mg_main_instance(NULL);
int PMain::mg_pid(getpid());
FILE *PMain::mg_out_file(stdout);
FILE *PMain::mg_err_file(stderr);
const char *PMain::mg_proc_name("");
bool PMain::mg_verbose(false);


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
size_t PMain::ArrayLength(const void * const * const a_array)
//====================================================================
{
	size_t len;
	
	for (len = 0; a_array[len] != NULL; len++)
	{
		;
	}
	
	return len;
}

//====================================================================
double PMain::BoundRandom(const double a_lo, const double a_hi)
//====================================================================
{
	int random_number;
	
	// Return a number between 0 (inclusive) and 1 (exclusive)
	// Note: in order to avoid low-balled results, we must pick
	// another random number if random() returns INT_MAX
	while ((random_number = random()) == INT_MAX)
	{
		;
	}
	
	return (a_hi - a_lo) * random_number / INT_MAX + a_lo;
}

//====================================================================
void PMain::Err(const bool a_prnt_proc_name, const char * const a_fmt, ...)
//====================================================================
{
	// Get the arguments and send them to the presses
	va_list args;
	va_start(args, a_fmt);
	Print(mg_err_file, a_prnt_proc_name, a_fmt, args);
	va_end(args);
}

//====================================================================
void PMain::ErrDump(const bool a_prnt_proc_name, const char * const a_fmt, ...)
//====================================================================
{
	// Get the arguments and send them to the presses
	va_list args;
	va_start(args, a_fmt);
	Print(mg_err_file, a_prnt_proc_name, a_fmt, args);
	va_end(args);
	
	// Hasta lasagna, don't get any on ya
	abort();
}

//====================================================================
void PMain::ErrExit(const bool a_prnt_proc_name, const int a_err, const char * const a_fmt, ...)
//====================================================================
{
	// Get the arguments and send them to the presses
	va_list args;
	va_start(args, a_fmt);
	Print(mg_err_file, a_prnt_proc_name, a_fmt, args);
	va_end(args);
	
	// Bye
	Exit(a_err);
}

//====================================================================
FILE *PMain::ErrFile(FILE *a_err_file)
//====================================================================
{
	// Keep the old file
	FILE *old_err_file(mg_err_file);
	
	// Set the new one if there's one to set
	if (a_err_file != NULL)
	{
		mg_err_file = a_err_file;
	}
	
	return old_err_file;
}

//====================================================================
void PMain::Exit(const int a_err)
//====================================================================
{
	// (Badly) simulated semaphore
	static int num_exiting(0);
	num_exiting++;
	
	// Check to see if we're first in line
	if (num_exiting > 1)
	{
		// A day late and a dollar short
		num_exiting--;
		return;
	}
	// Call the destructor for PMain
	else
	{
		delete mg_main_instance;
	}
	
	// Fire in the hole!
	exit(a_err);
}

//====================================================================
void PMain::ExitNoMem(void)
//====================================================================
{
	ErrExit(true, mk_exit_code_no_memory, "out of memory\n");
}

//====================================================================
void PMain::Out(const bool a_prnt_proc_name, const char * const a_fmt, ...)
//====================================================================
{
	// Get the arguments and send them to the presses
	va_list args;
	va_start(args, a_fmt);
	Print(mg_out_file, a_prnt_proc_name, a_fmt, args);
	va_end(args);
}

//====================================================================
FILE *PMain::OutFile(FILE *a_out_file)
//====================================================================
{
	// Keep the old file
	FILE *old_out_file(mg_out_file);
	
	// Set the new one if there's one to set
	if (a_out_file != NULL)
	{
		mg_out_file = a_out_file;
	}
	
	return old_out_file;
}

//====================================================================
void PMain::Verbose(const bool a_prnt_proc_name, const char * const a_fmt, ...)
//====================================================================
{
	// Check to make sure we should print this
	if (!IsVerbose())
	{
		return;
	}
	
	// Get the arguments and send them to the presses
	va_list args;
	va_start(args, a_fmt);
	Print(mg_out_file, a_prnt_proc_name, a_fmt, args);
	va_end(args);
}

//====================================================================
PMain::PMain(int &a_argc, const char ** const a_argv) :
//====================================================================
	m_argc(a_argc),
	m_argv(a_argv),
	m_ready_to_run(false),
	m_running(false)
{
	// We're trying to simulate semaphores here, but there is room
	// for error
	static int num_instances(0);
	num_instances++;
	
	// Check to see if we're first in line
	if (num_instances > 1)
	{
		// A day late and a dollar short
		num_instances--;
		return;
	}
	
	// We're fine; initialize the first instance
	mg_main_instance = this;
	m_ready_to_run = true;
	
	// Get rid of any path in the process name
	mg_proc_name = strrchr(a_argv[0], mk_path_delim);
	
	if (mg_proc_name == NULL)
	{
		mg_proc_name = a_argv[0];
	}
	else
	{
		mg_proc_name++;
	}
	
	// Initialize the random seeds
	srand((time(NULL) * mg_pid) | 0x00000001);
	srandom((time(NULL) * mg_pid) | 0x00000001);
}

//====================================================================
PMain::~PMain(void)
//====================================================================
{
	if (mg_main_instance == this)
	{
		mg_main_instance = NULL;
	}
}

//====================================================================
bool PMain::Start(void)
//====================================================================
{
	// (Badly) simulated semaphore
	static int num_running(0);
	num_running++;
	
	// Check to see if we're first in line
	if (num_running > 1)
	{
		// A day late and a dollar short
		num_running--;
		return false;
	}
	
	// Barf in not ready to run
	if (!IsReadyToRun())
	{
		return false;
	}
	
	// Fire in the hole!
	Run(m_argc, m_argv);
	
	return true;
}

//====================================================================
const char *PMain::ShiftArgs(int &a_argc, const char ** const a_argv, const int a_which)
//====================================================================
{
	// Make sure we can shift the arguments
	if (a_which >= a_argc)
	{
		return NULL;
	}
	
	// We'll end up with one less than we started
	a_argc--;
	
	const char *arg(a_argv[a_which]);
	
	// Move everything down
	for (int i(a_which); i < a_argc; i++)
	{
		a_argv[i] = a_argv[i + 1];
	}
	
	// Maintain POSIX compatibility
	a_argv[a_argc] = NULL;
	
	return arg;
}

//====================================================================
void PMain::Print(FILE *a_file, const bool a_prnt_proc_name, const char * const a_fmt, const va_list a_args)
//====================================================================
{
	// Check to see if we should print the process name first
	if (a_prnt_proc_name)
	{
		fprintf(a_file, "%s: ", mg_proc_name);
	}
	
	// Send it to the presses
	vfprintf(a_file, a_fmt, a_args);
	fflush(a_file);
}
