/*--------------------------------------------------------------------*\
  File:      PMainLttr.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      PMainLttr class.
  ID:        $Id: PMainLttr.cpp,v 1.24 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "PMainLttr.h"
#include <string.h>


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the PMainLttr protected static data members
const char PMainLttr::mk_opt_esc('-');
const char * const PMainLttr::mk_word_opt_delim("=");
const char * const PMainLttr::mk_word_arg_delim(",");

// Initialize the PMainLttr private static data members
const char * const PMainLttr::mk_lttr_opts("?vV");

const char * const PMainLttr::mk_word_opts[] = 
{
	"help",
	"verbose",
	"version",
	NULL
};

const size_t PMainLttr::mk_opt_args[] = 
{
	0,    // '?', "help"
	0,    // 'v', "verbose"
	0     // 'V', "version"
};

const size_t PMainLttr::mk_opt_help_index(0);
const size_t PMainLttr::mk_opt_vrbs_index(PMainLttr::mk_opt_help_index + 1);
const size_t PMainLttr::mk_opt_vers_index(PMainLttr::mk_opt_vrbs_index + 1);


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
PMainLttr::PMainLttr(int &a_argc, const char ** const a_argv, const char * const a_lttr_opts, const size_t * const a_lttr_opt_args, const char * const * const a_word_opts, const size_t * const a_word_opt_args) :
//====================================================================
	Inherited(a_argc, a_argv),
	m_print_help(false),
	m_print_vers(false)
{
	size_t i;
	
	// Allocate memory to hold the letter options
	size_t num_known_lttr_opts(strlen(mk_lttr_opts));
	size_t num_lttr_opts(num_known_lttr_opts + ((a_lttr_opts != NULL) ? strlen(a_lttr_opts) : 0));
	size_t num_known_word_opts(PMain::ArrayLength(reinterpret_cast<const void * const * const>(mk_word_opts)));
	size_t num_word_opts(num_known_word_opts + ((a_word_opts != NULL) ? PMain::ArrayLength(reinterpret_cast<const void * const * const>(a_word_opts)) : 0));
	
	// Check for a memory allocation error
	if ((m_lttr_opts = new char[num_lttr_opts + 1]) == NULL
		|| (m_lttr_opt_args = new size_t[num_lttr_opts]) == NULL
		|| (m_word_opts = new char*[num_word_opts + 1]) == NULL
		|| (m_word_opt_args = new size_t[num_word_opts]) == NULL)
	{
		delete m_lttr_opts;
		delete m_word_opts;
		delete m_lttr_opt_args, m_word_opt_args;
		ExitNoMem();
	}
	
	for (i = 0; i <= num_word_opts; i++)
	{
		m_word_opts[i] = NULL;
	}
	
	// Copy the letter options array
	strcpy(m_lttr_opts, mk_lttr_opts);
	
	if (a_lttr_opts != NULL)
	{
		strcat(m_lttr_opts, a_lttr_opts);
	}
	
	// Copy the word options array
	for (i = 0; i < num_known_word_opts; i++)
	{
		// Check for a memory allocation error
		if ((m_word_opts[i] = strdup(mk_word_opts[i])) == NULL)
		{
			ExitNoMem();
		}
	}
	
	for (; i < num_word_opts; i++)
	{
		// Check for a memory allocation error
		if ((m_word_opts[i] = strdup(a_word_opts[i - num_known_word_opts])) == NULL)
		{
			ExitNoMem();
		}
	}
	
	// Copy the option arguments arrays
	for (i = 0; i < num_known_lttr_opts; i++)
	{
		m_lttr_opt_args[i] = mk_opt_args[i];
	}
	
	for (; i < num_lttr_opts; i++)
	{
		m_lttr_opt_args[i] = a_lttr_opt_args[i - num_known_lttr_opts];
	}
	
	for (i = 0; i < num_known_word_opts; i++)
	{
		m_word_opt_args[i] = mk_opt_args[i];
	}
	
	for (; i < num_word_opts; i++)
	{
		m_word_opt_args[i] = a_word_opt_args[i - num_known_word_opts];
	}
}

//====================================================================
PMainLttr::~PMainLttr(void)
//====================================================================
{
	delete [] m_lttr_opts;
	delete [] m_lttr_opt_args;
	
	if (m_word_opts != NULL)
	{
		for (size_t i(0); m_word_opts[i] != NULL; i++)
		{
			delete [] m_word_opts[i];
		}
	}
	
	delete [] m_word_opts;
	delete [] m_word_opt_args;
}

//====================================================================
size_t PMainLttr::HandleLttrOpt(const char a_opt, const char ** const /*a_opt_args*/)
//====================================================================
{
	Err(true, "unknown option -- `%c'\n", a_opt);
	
	return 1;
}

//====================================================================
size_t PMainLttr::HandleWordOpt(const char * const a_opt, const char ** const /*a_opt_args*/)
//====================================================================
{
	Err(true, "unknown option -- `%s'\n", a_opt);
	
	return 1;
}

//====================================================================
void PMainLttr::Help(void) const
//====================================================================
{
	Inherited::Help();
	Out(false, "\nOPTIONS\n");
	Out(false, "\n%c%c, %c%c%s\n", mk_opt_esc, mk_lttr_opts[mk_opt_vrbs_index], mk_opt_esc, mk_opt_esc, mk_word_opts[mk_opt_vrbs_index]);
	Out(false, "    Output additional diagnostics to stdout.\n");
	Out(false, "\n%c%c, %c%c%s\n", mk_opt_esc, mk_lttr_opts[mk_opt_help_index], mk_opt_esc, mk_opt_esc, mk_word_opts[mk_opt_help_index]);
	Out(false, "    Display this help and exit.\n");
	Out(false, "\n%c%c, %c%c%s\n", mk_opt_esc, mk_lttr_opts[mk_opt_vers_index], mk_opt_esc, mk_opt_esc, mk_word_opts[mk_opt_vers_index]);
	Out(false, "    Output version information and exit.\n");
}

//====================================================================
int PMainLttr::ParseOpts(int &a_argc, const char ** const a_argv)
//====================================================================
{
	size_t errs(0);
	
	// Get all the options
	for (size_t i(mk_first_arg_ind); i < a_argc;)
	{
		// Continue if it's not an option
		if (a_argv[i][0] != mk_opt_esc)
		{
			i++;
			continue;
		}
		
		// Get rid of the option escape character
		const char *cur_arg(ShiftArgs(a_argc, a_argv, i));
		cur_arg++;
		
		// Handle word-based options
		if (*cur_arg == mk_opt_esc)
		{
			errs += ParseWordOpt(++cur_arg);
		}
		// Handle letter-based options
		else
		{
			errs += ParseLttrOpts(a_argc, a_argv, cur_arg, i);
		}
	}
	
	// Display an error if necessary
	if (errs > 0)
	{
		Err(true, "%d command line parse error%s\n", errs, (errs != 1) ? "s" : "");
		Err(true, "try `%s %c%c' or `%s %c%c%s' for assistance\n", ProcName(), mk_opt_esc, mk_lttr_opts[mk_opt_help_index], ProcName(), mk_opt_esc, mk_opt_esc, mk_word_opts[mk_opt_help_index]);
		return mk_exit_code_bad_args;
	}
	// Print the help and exit if required
	else if (m_print_help)
	{
		Help();
		Exit(mk_exit_code_no_err);
	}
	// Print the version and exit if required
	else if (m_print_vers)
	{
		Version();
		Exit(mk_exit_code_no_err);
	}
	
	return mk_exit_code_no_err;
}

//====================================================================
void PMainLttr::Usage(void) const
//====================================================================
{
	Out(false, "Usage: %s [ OPTION ] ... [ ARG ] ...\n", ProcName());
}

//====================================================================
void PMainLttr::Version(void) const
//====================================================================
{
	Out(false, "%s, version 1.0\n", ProcName());
}

//====================================================================
size_t PMainLttr::HandleKnownLttrOpt(const char a_opt, const char ** const a_opt_args)
//====================================================================
{
	// Handle the help letter-based option
	if (a_opt == mk_lttr_opts[mk_opt_help_index])
	{
		m_print_help = true;
	}
	// Handle the verbose letter-based option
	else if (a_opt == mk_lttr_opts[mk_opt_vrbs_index])
	{
		SetVerbose(true);
	}
	// Handle the version letter-based option
	else if (a_opt == mk_lttr_opts[mk_opt_vers_index])
	{
		m_print_vers = true;
	}
	// Handle the unknown letter-based options
	else
	{
		return HandleLttrOpt(a_opt, a_opt_args);
	}
	
	return 0;
}

//====================================================================
size_t PMainLttr::HandleKnownWordOpt(const char * const a_opt, const char ** const a_opt_args)
//====================================================================
{
	// Handle the help word-based option
	if (strcmp(a_opt, mk_word_opts[mk_opt_help_index]) == 0)
	{
		m_print_help = true;
	}
	// Handle the verbose word-based option
	else if (strcmp(a_opt, mk_word_opts[mk_opt_vrbs_index]) == 0)
	{
		SetVerbose(true);
	}
	// Handle the version word-based option
	else if (strcmp(a_opt, mk_word_opts[mk_opt_vers_index]) == 0)
	{
		m_print_vers = true;
	}
	// Handle the unknown word-based options
	else
	{
		return HandleWordOpt(a_opt, a_opt_args);
	}
	
	return 0;
}

//====================================================================
size_t PMainLttr::ParseLttrOpts(int &a_argc, const char ** const a_argv, const char * const a_arg, const size_t a_index)
//====================================================================
{
	size_t errs(0);
	
	// Check the length
	if (*a_arg == '\0')
	{
		Err(true, "null option after `%c'\n", mk_opt_esc);
		return ++errs;
	}
	
	const char *cur_arg(a_arg);
	
	// Deal with the letter options
	while (*cur_arg != '\0')
	{
		// Get the option and its index
		char opt(*cur_arg);
		size_t opt_ind(strchr(m_lttr_opts, opt) - m_lttr_opts);
		
		// Give error if it isn't known
		if (opt_ind == -static_cast<int>(m_lttr_opts))
		{
			Err(true, "unknown option -- `%c'\n", opt);
			errs++;
			cur_arg++;
			continue;
		}
		
		// Find the number of arguments required for the
		// option
		size_t num_opt_args(m_lttr_opt_args[opt_ind]);
		
		// Handle the option
		if (num_opt_args == 0)
		{
			cur_arg++;
			errs += HandleKnownLttrOpt(opt, NULL);
		}
		// Get the arguments
		else
		{
			size_t i(0);
			const char ** opt_argv;
			
			// Check for a memory allocation error
			if ((opt_argv = new const char *[num_opt_args]) == NULL)
			{
				ExitNoMem();
			}
			
			// See if the first argument follows directly
			// after the option
			opt_argv[i] = (*(++cur_arg) != '\0') ? cur_arg : ShiftArgs(a_argc, a_argv, a_index);
			cur_arg += strlen(cur_arg);
			
			// Check to make sure we got one
			if (opt_argv[i++] == NULL)
			{
				Err(true, "too few arguments to option -- `%c'\n", opt);
				errs++;
				delete [] opt_argv;
				continue;
			}
			
			// Get the rest of the arguments (if any)
			for (; i < num_opt_args; i++)
			{
				opt_argv[i] = ShiftArgs(a_argc, a_argv, a_index);
				
				// Check to make sure we got one
				if (opt_argv[i] == NULL)
				{
					Err(true, "too few arguments to option -- `%c'\n", opt);
					errs++;
					delete [] opt_argv;
					break;
				}
			}
			
			// We finally get to do something with the
			// option (maybe)
			if (i == num_opt_args)
			{
				errs += HandleKnownLttrOpt(opt, opt_argv);
			}
			
			delete [] opt_argv;
		}
	}
	
	return errs;
}

//====================================================================
size_t PMainLttr::ParseWordOpt(const char * const a_arg)
//====================================================================
{
	size_t errs(0);
	
	// Make a copy of the current argument
	char *cur_arg;
	
	// Check for an allocation error
	if ((cur_arg = strdup(a_arg)) == NULL)
	{
		ExitNoMem();
	}
	
	char *arg, *opt(strtok(cur_arg, mk_word_opt_delim));
	
	// Check the option
	if (opt == NULL)
	{
		Err(true, "null option after `%c%c'\n", mk_opt_esc, mk_opt_esc);
		delete [] cur_arg;
		return ++errs;
	}
	
	// Find out if we know about this one and get the index
	size_t opt_ind(0);
	
	while (m_word_opts[opt_ind] != NULL)
	{
		if (strcmp(m_word_opts[opt_ind], opt) == 0)
		{
			break;
		}
		else
		{
			opt_ind++;
		}
	}
	
	// Check to see if we found a match
	if (m_word_opts[opt_ind] == NULL)
	{
		Err(true, "unknown option -- `%s'\n", opt);
		delete [] cur_arg;
		return ++errs;
	}
	
	// Find the number of arguments required for the option
	size_t i(0), num_opt_args(m_word_opt_args[opt_ind]);
	const char ** opt_argv(NULL);
	
	// Check for a memory allocation error
	if (num_opt_args > 0
		&& (opt_argv = new const char *[num_opt_args]) == NULL)
	{
		delete [] cur_arg;
		ExitNoMem();
	}
	
	while ((arg = strtok(NULL, mk_word_arg_delim)) != NULL)
	{
		// Check to make sure we don't have too many
		if (i >= num_opt_args)
		{
			Err(true, "too many arguments to option -- `%s'\n", opt);
			delete [] cur_arg;
			delete [] opt_argv;
			return ++errs;
		}
		
		opt_argv[i++] = arg;
	}
	
	// Check to make sure we got enough
	if (i < num_opt_args)
	{
		Err(true, "too few arguments to option -- `%s'\n", opt);
		delete [] cur_arg;
		delete [] opt_argv;
		return ++errs;
	}
	
	// We finally get to do something with the option (maybe)
	errs += HandleKnownWordOpt(opt, opt_argv);
	delete [] cur_arg;
	delete [] opt_argv;
	
	return errs;
}
