/*--------------------------------------------------------------------*\
  File:      PMainLttr.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the PMainLttr abstract class.
  ID:        $Id: PMainLttr.h,v 1.17 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "PMain.h"


#ifndef P_MAIN_LTTR_H
#define P_MAIN_LTTR_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifndef DECL_SPEC

#if defined DLL_EXPORT_SYMBOLS
#define DECL_SPEC __declspec(dllexport)
#elif defined DLL_IMPORT_SYMBOLS
#define DECL_SPEC __declspec(dllimport)
#else
#define DECL_SPEC
#endif

#endif    // DECL_SPEC


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifdef DLL_EXPORT_SYMBOLS
#pragma export on
#endif

/*--------------------------------------------------------------------*\
  Class name:       PMainLttr
  Inherits from:    public PMain
  New data members: protected static const char mk_opt_esc - the
                        escape character used for the command line
                        options.
                    protected static const char mk_word_opt_delim -
                        the delimiter for the word-based options and
                        their arguments.
                    protected static const char mk_word_arg_delim -
                        the delimiter for the word-based options'
                        arguments.
                    private static const char * const mk_lttr_opts -
                        the known letter-based options.
                    private static const size_t mk_lttr_opt_args[] -
                        the known letter-based options' arguments.
                    private static const char * const mk_word_opts[] -
                        the known word-based options.
                    private static const size_t mk_opt_args[] - the
                        known options' arguments.
                    private static const size_t mk_opt_help_index -
                        the index of the help command line option.
                    private static const size_t mk_opt_vrbs_index -
                        the index of the verbose command line option.
                    private static const size_t mk_opt_vers_index -
                        the index of the version command line option.
                    private char *m_lttr_opts - a concatenation of the
                        known and unknown letter-based command line
                        options.
                    private size_t *m_lttr_opt_args - a concatenation
                        of the known and unknown letter-based command
                        line options' arguments.
                    private char *m_word_opts - a concatenation of the
                        known and unknown word-based command line
                        options.
                    private size_t *m_word_opt_args - a concatenation
                        of the known and unknown word-based command
                        line options' arguments.
                    private bool m_print_help - if this is set to true
                        during the ParseOpts() function, the Help()
                        method will be called, and the program will
                        exit before reaching the Run() method.
                    private bool m_print_vers - if this is set to true
                        during the ParseOpts() function, the Version()
                        method will be called, and the program will
                        exit before reaching the Run() method.
  Description: Abstract class to implement a PMain with a letter-based
      command line option parsing scheme.
\*--------------------------------------------------------------------*/

class DECL_SPEC PMainLttr : public PMain
{
	public:
	
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: PMainLttr
  Member of:     public PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
                 const char * const a_lttr_opts - a subclass-defined
                     string containing each possible letter-based
                     command line option. Default: NULL.
                 const size_t * const a_lttr_opt_args - a subclass-
                     defined array containing the number of arguments
                     for each command line option defined in
                     a_lttr_opts. Default: NULL.
                 const char * const * const a_word_opts - a subclass-
                     defined, null-terminated array of strings
                     containing each possible word-based command line
                     option. Default: NULL.
                 const size_t * const a_word_opt_args - a subclass-
                     defined array containing the number of arguments
                     for each command line option defined in
                     a_word_opts. Default: NULL.
  Throws:        none
  Description: Class constructor.
\*--------------------------------------------------------------------*/
		
		PMainLttr(int &a_argc, const char ** const a_argv, const char * const a_lttr_opts = NULL, const size_t * const a_lttr_opt_args = NULL, const char * const * const a_word_opts = NULL, const size_t * const a_word_opt_args = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~PMainLttr
  Member of:     public PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~PMainLttr(void);
	
	protected:
	
		// Protected static const data members
		static const char mk_opt_esc;
		static const char * const mk_word_opt_delim;
		static const char * const mk_word_arg_delim;
		
		// Protected member functions
		
/*--------------------------------------------------------------------*\
  Function name: virtual HandleLttrOpt
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     const char a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Hook function called to handle the arguments not
      handled by HandleKnownLttrOpt().
\*--------------------------------------------------------------------*/
		
		virtual size_t HandleLttrOpt(const char a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: virtual HandleWordOpt
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     const char * const a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Hook function called to handle the arguments not
      handled by HandleKnownWordOpt().
\*--------------------------------------------------------------------*/
		
		virtual size_t HandleWordOpt(const char * const a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: virtual Help const
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Help(void) const;
		
/*--------------------------------------------------------------------*\
  Function name: virtual ParseOpts
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
  Returns:       int - the error code resulting from the parsing of
                     the arguments (zero if there was no error).
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual int ParseOpts(int &a_argc, const char ** const a_argv);
		
/*--------------------------------------------------------------------*\
  Function name: virtual Usage const
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Usage(void) const;
		
/*--------------------------------------------------------------------*\
  Function name: virtual Version const
  Member of:     protected PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Version(void) const;
	
	private:
	
		typedef PMain Inherited;
		
		// Private static const data members
		static const char * const mk_lttr_opts;
		static const char * const mk_word_opts[];
		static const size_t mk_opt_args[];
		static const size_t mk_opt_help_index;
		static const size_t mk_opt_vrbs_index;
		static const size_t mk_opt_vers_index;
		
		// Private data members
		char *m_lttr_opts;
		size_t *m_lttr_opt_args;
		char **m_word_opts;
		size_t *m_word_opt_args;
		
		bool m_print_help;
		bool m_print_vers;
		
		// Private member functions
		
/*--------------------------------------------------------------------*\
  Function name: HandleKnownLttrOpt
  Member of:     private PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     const char a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Function to handle the known letter-based arguments.
\*--------------------------------------------------------------------*/
		
		size_t HandleKnownLttrOpt(const char a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: HandleKnownWordOpt
  Member of:     private PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     const char * const a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Function to handle the known word-based arguments.
\*--------------------------------------------------------------------*/
		
		size_t HandleKnownWordOpt(const char * const a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: ParseLttrOpts
  Member of:     private PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
                 const char * const a_arg - the argument which is
                     currently being parsed.
                 const size_t a_index - the index of the argument
                     which is currently being parsed.
  Returns:       size_t - the number of errors resulting from the
                     parsing of the arguments.
  Throws:        none
  Description: Function to parse the letter-based options from a given
      command line argument.
\*--------------------------------------------------------------------*/
		
		size_t ParseLttrOpts(int &a_argc, const char ** const a_argv, const char * const a_arg, const size_t a_index);
		
/*--------------------------------------------------------------------*\
  Function name: ParseWordOpt
  Member of:     private PMainLttr
  Defined in:    PMainLttr.cpp
  Arguments:     const char * const a_arg - the argument which is
                     currently being parsed.
  Returns:       size_t - the number of errors resulting from the
                     parsing of the arguments.
  Throws:        none
  Description: Function to parse a given word-based option.
\*--------------------------------------------------------------------*/
		
		size_t ParseWordOpt(const char * const a_arg);
};

#ifdef DLL_EXPORT_SYMBOLS
#pragma export off
#endif


#endif    // P_MAIN_LTTR_H
