/*--------------------------------------------------------------------*\
  File:      PMainWord.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      PMainWord class.
  ID:        $Id: PMainWord.cpp,v 1.12 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "PMainWord.h"
#include <string.h>


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the PMainWord protected static data members
const char PMainWord::mk_opt_esc('-');

// Initialize the PMainWord private static data members
const char * const PMainWord::mk_opts[] = 
{
	"?",
	"verbose",
	"version",
	NULL
};

const size_t PMainWord::mk_opt_args[] = 
{
	0,    // "?"
	0,    // "verbose"
	0     // "version"
};

const size_t PMainWord::mk_opt_help_index(0);
const size_t PMainWord::mk_opt_vrbs_index(PMainWord::mk_opt_help_index + 1);
const size_t PMainWord::mk_opt_vers_index(PMainWord::mk_opt_vrbs_index + 1);


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
PMainWord::PMainWord(int &a_argc, const char ** const a_argv, const char * const * const a_opts, const size_t * const a_opt_args) :
//====================================================================
	Inherited(a_argc, a_argv),
	m_print_help(false),
	m_print_vers(false)
{
	size_t i;
	
	// Allocate memory to hold the options
	size_t num_known_opts(PMain::ArrayLength(reinterpret_cast<const void * const * const>(mk_opts)));
	size_t num_opts(num_known_opts + ((a_opts != NULL) ? PMain::ArrayLength(reinterpret_cast<const void * const * const>(a_opts)) : 0));
	
	// Check for a memory allocation error
	if ((m_opts = new char*[num_opts + 1]) == NULL
		|| (m_opt_args = new size_t[num_opts]) == NULL)
	{
		delete m_opts;
		delete m_opt_args;
		ExitNoMem();
	}
	
	for (i = 0; i <= num_opts; i++)
	{
		m_opts[i] = NULL;
	}
	
	// Copy the options array
	for (i = 0; i < num_known_opts; i++)
	{
		// Check for a memory allocation error
		if ((m_opts[i] = strdup(mk_opts[i])) == NULL)
		{
			ExitNoMem();
		}
	}
	
	for (; i < num_opts; i++)
	{
		// Check for a memory allocation error
		if ((m_opts[i] = strdup(a_opts[i - num_known_opts])) == NULL)
		{
			ExitNoMem();
		}
	}
	
	// Copy the option arguments arrays
	for (i = 0; i < num_known_opts; i++)
	{
		m_opt_args[i] = mk_opt_args[i];
	}
	
	for (; i < num_opts; i++)
	{
		m_opt_args[i] = a_opt_args[i - num_known_opts];
	}
}

//====================================================================
PMainWord::~PMainWord(void)
//====================================================================
{
	if (m_opts != NULL)
	{
		for (size_t i(0); m_opts[i] != NULL; i++)
		{
			delete [] m_opts[i];
		}
	}
	
	delete [] m_opts;
	delete [] m_opt_args;
}

//====================================================================
size_t PMainWord::HandleWordOpt(const char * const a_opt, const char ** const /*a_opt_args*/)
//====================================================================
{
	Err(true, "unknown option -- `%s'\n", a_opt);
	
	return 1;
}

//====================================================================
void PMainWord::Help(void) const
//====================================================================
{
	Inherited::Help();
	Out(false, "\nOPTIONS\n");
	Out(false, "\n%c%s\n", mk_opt_esc, mk_opts[mk_opt_vrbs_index]);
	Out(false, "    Output additional diagnostics to stdout.\n");
	Out(false, "\n%c%s\n", mk_opt_esc, mk_opts[mk_opt_help_index]);
	Out(false, "    Display this help and exit.\n");
	Out(false, "\n%c%s\n", mk_opt_esc, mk_opts[mk_opt_vers_index]);
	Out(false, "    Output version information and exit.\n");
}

//====================================================================
int PMainWord::ParseOpts(int &a_argc, const char ** const a_argv)
//====================================================================
{
	size_t errs(0);
	
	// Get all the options
	for (size_t i(mk_first_arg_ind); i < a_argc;)
	{
		// Continue if it's not an option
		if (a_argv[i][0] != mk_opt_esc)
		{
			i++;
			continue;
		}
		
		// Get rid of the option escape character
		const char *cur_arg(ShiftArgs(a_argc, a_argv, i));
		cur_arg++;
		
		// Handle option
		errs += ParseWordOpt(a_argc, a_argv, cur_arg, i);
	}
	
	// Display an error if necessary
	if (errs > 0)
	{
		Err(true, "%d command line parse error%s\n", errs, (errs != 1) ? "s" : "");
		Err(true, "try `%s %c%s' for assistance\n", ProcName(), mk_opt_esc, mk_opts[mk_opt_help_index]);
		return mk_exit_code_bad_args;
	}
	// Print the help and exit if required
	else if (m_print_help)
	{
		Help();
		Exit(mk_exit_code_no_err);
	}
	// Print the version and exit if required
	else if (m_print_vers)
	{
		Version();
		Exit(mk_exit_code_no_err);
	}
	
	return mk_exit_code_no_err;
}

//====================================================================
void PMainWord::Usage(void) const
//====================================================================
{
	Out(false, "Usage: %s [ OPTION ] ... [ ARG ] ...\n", ProcName());
}

//====================================================================
void PMainWord::Version(void) const
//====================================================================
{
	Out(false, "%s, version 1.0\n", ProcName());
}

//====================================================================
size_t PMainWord::HandleKnownWordOpt(const char * const a_opt, const char ** const a_opt_args)
//====================================================================
{
	// Handle the help option
	if (strcmp(a_opt, mk_opts[mk_opt_help_index]) == 0)
	{
		m_print_help = true;
	}
	// Handle the verbose option
	else if (strcmp(a_opt, mk_opts[mk_opt_vrbs_index]) == 0)
	{
		SetVerbose(true);
	}
	// Handle the version option
	else if (strcmp(a_opt, mk_opts[mk_opt_vers_index]) == 0)
	{
		m_print_vers = true;
	}
	// Handle the unknown options
	else
	{
		return HandleWordOpt(a_opt, a_opt_args);
	}
	
	return 0;
}

//====================================================================
size_t PMainWord::ParseWordOpt(int &a_argc, const char ** const a_argv, const char * const a_arg, const size_t a_index)
//====================================================================
{
	size_t errs(0);
	
	// Check the length
	if (*a_arg == '\0')
	{
		Err(true, "null option after `%c'\n", mk_opt_esc);
		return ++errs;
	}
	
	// Get the option and its index
	size_t opt_ind;
	
	for (opt_ind = 0; m_opts[opt_ind] != NULL; opt_ind++)
	{
		if (strcmp(a_arg, m_opts[opt_ind]) == 0)
		{
			break;
		}
	}
	
	// Give error if it isn't known
	if (m_opts[opt_ind] == NULL)
	{
		Err(true, "unknown option -- `%s'\n", a_arg);
		return 1;
	}
	
	// Find the number of arguments required for the option
	size_t num_opt_args(m_opt_args[opt_ind]);
	
	// Handle the option
	if (num_opt_args == 0)
	{
		return HandleKnownWordOpt(a_arg, NULL);
	}
	
	// Get the arguments
	const char ** opt_argv;
	
	// Check for a memory allocation error
	if ((opt_argv = new const char *[num_opt_args]) == NULL)
	{
		ExitNoMem();
	}
	
	// Get the arguments
	size_t i;
	
	for (i = 0; i < num_opt_args; i++)
	{
		opt_argv[i] = ShiftArgs(a_argc, a_argv, a_index);
		
		// Check to make sure we got one
		if (opt_argv[i] == NULL)
		{
			Err(true, "too few arguments to option -- `%s'\n", a_arg);
			delete [] opt_argv;
			return 1;
		}
	}
	
	// We finally get to do something with the option (maybe)
	if (i == num_opt_args)
	{
		return HandleKnownWordOpt(a_arg, opt_argv);
	}
	
	delete [] opt_argv;
	
	return 0;
}
