/*--------------------------------------------------------------------*\
  File:      PMainWord.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Header describing the PMainWord abstract class.
  ID:        $Id: PMainWord.h,v 1.12 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "PMain.h"


#ifndef P_MAIN_WORD_H
#define P_MAIN_WORD_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifndef DECL_SPEC

#if defined DLL_EXPORT_SYMBOLS
#define DECL_SPEC __declspec(dllexport)
#elif defined DLL_IMPORT_SYMBOLS
#define DECL_SPEC __declspec(dllimport)
#else
#define DECL_SPEC
#endif

#endif    // DECL_SPEC


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifdef DLL_EXPORT_SYMBOLS
#pragma export on
#endif

/*--------------------------------------------------------------------*\
  Class name:       PMainWord
  Inherits from:    public PMain
  New data members: protected static const char mk_opt_esc - the
                        escape character used for the command line
                        options.
                    private static const char * const mk_opts[] - the
                        known options.
                    private static const size_t mk_opt_args[] - the
                        known options' arguments.
                    private static const size_t mk_opt_help_index -
                        the index of the help command line option.
                    private static const size_t mk_opt_vrbs_index -
                        the index of the verbose command line option.
                    private static const size_t mk_opt_vers_index -
                        the index of the version command line option.
                    private char *m_opts - a concatenation of the
                        known and unknown command line options.
                    private size_t *m_opt_args - a concatenation of
                        the known and unknown command line options'
                        arguments.
                    private bool m_print_help - if this is set to true
                        during the ParseOpts() function, the Help()
                        method will be called, and the program will
                        exit before reaching the Run() method.
                    private bool m_print_vers - if this is set to true
                        during the ParseOpts() function, the Version()
                        method will be called, and the program will
                        exit before reaching the Run() method.
  Description: Abstract class to implement a PMain with a word-based
      command line option parsing scheme.
\*--------------------------------------------------------------------*/

class DECL_SPEC PMainWord : public PMain
{
	public:
	
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: PMainWord
  Member of:     public PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
                 const char * const * const a_opts - a subclass-
                     defined, null-terminated array of strings
                     containing each possible command line option.
                     Default: NULL.
                 const size_t * const a_opt_args - a subclass-defined
                     array containing the number of arguments for each
                     command line option defined in a_opts. Default:
                     NULL.
  Throws:        none
  Description: Class constructor.
\*--------------------------------------------------------------------*/
		
		PMainWord(int &a_argc, const char ** const a_argv, const char * const * const a_opts = NULL, const size_t * const a_opt_args = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~PMainWord
  Member of:     public PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~PMainWord(void);
	
	protected:
	
		// Protected static const data members
		static const char mk_opt_esc;
		
		// Protected member functions
		
/*--------------------------------------------------------------------*\
  Function name: virtual HandleWordOpt
  Member of:     protected PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     const char * const a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Hook function called to handle the arguments not
      handled by HandleKnownWordOpt().
\*--------------------------------------------------------------------*/
		
		virtual size_t HandleWordOpt(const char * const a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: virtual Help const
  Member of:     protected PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Help(void) const;
		
/*--------------------------------------------------------------------*\
  Function name: virtual ParseOpts
  Member of:     protected PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
  Returns:       int - the error code resulting from the parsing of
                     the arguments (zero if there was no error).
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual int ParseOpts(int &a_argc, const char ** const a_argv);
		
/*--------------------------------------------------------------------*\
  Function name: virtual Usage const
  Member of:     protected PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Usage(void) const;
		
/*--------------------------------------------------------------------*\
  Function name: virtual Version const
  Member of:     protected PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Version(void) const;
	
	private:
	
		typedef PMain Inherited;
		
		// Private static const data members
		static const char * const mk_opts[];
		static const size_t mk_opt_args[];
		static const size_t mk_opt_help_index;
		static const size_t mk_opt_vrbs_index;
		static const size_t mk_opt_vers_index;
		
		// Private data members
		char **m_opts;
		size_t *m_opt_args;
		
		bool m_print_help;
		bool m_print_vers;
		
		// Private member functions
		
/*--------------------------------------------------------------------*\
  Function name: HandleKnownWordOpt
  Member of:     private PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     const char * const a_opt - the option.
                 const char ** const a_opt_args - the option's
                     arguments.
  Returns:       size_t - the number of errors encountered when
                     handling the option and its arguments.
  Throws:        none
  Description: Function to handle the known arguments.
\*--------------------------------------------------------------------*/
		
		size_t HandleKnownWordOpt(const char * const a_opt, const char ** const a_opt_args);
		
/*--------------------------------------------------------------------*\
  Function name: ParseWordOpt
  Member of:     private PMainWord
  Defined in:    PMainWord.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
                 const char * const a_arg - the argument which is
                     currently being parsed.
                 const size_t a_index - the index of the argument
                     which is currently being parsed.
  Returns:       size_t - the number of errors resulting from the
                     parsing of the arguments.
  Throws:        none
  Description: Function to parse the options from a given command line
      argument.
\*--------------------------------------------------------------------*/
		
		size_t ParseWordOpt(int &a_argc, const char ** const a_argv, const char * const a_arg, const size_t a_index);
};

#ifdef DLL_EXPORT_SYMBOLS
#pragma export off
#endif


#endif    // P_MAIN_WORD_H
