/*--------------------------------------------------------------------*\
  File:      View.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the View class.
  ID:        $Id: View.h,v 1.32 1998/07/20 05:16:46 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <View.h>


#ifndef VIEW_H
#define VIEW_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
  Class name:       View
  Inherits from:    public BView
  New data members: private status_t m_status - the status of the
                        object.
  Description: Class to implement a view in which a constant string is
      constantly displayed.
\*--------------------------------------------------------------------*/

class View : public BView
{
	public:
	
/*--------------------------------------------------------------------*\
  Function name: View
  Member of:     public View
  Defined in:    View.cpp
  Arguments:     const BRect a_frame - the frame of the view.
                 const char * const a_name - the name of the view.
                 status_t * const a_err - a placeholder for the error
                     encountered during construction (see below).
                     Default: NULL.
  Throws:        none
  Description: Class constructor. You must check the value of a_err or
      the return value of Status() after invoking the constructor to
      see if the object is usable. See the Status() function (below)
      for possible error values.
\*--------------------------------------------------------------------*/
		
		View(const BRect a_frame, const char * const a_name, status_t * const a_err = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~View
  Member of:     public View
  Defined in:    View.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~View(void);
		
/*--------------------------------------------------------------------*\
  Function name: virtual AttachedToWindow
  Member of:     public View
  Defined in:    View.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void AttachedToWindow(void);
		
/*--------------------------------------------------------------------*\
  Function name: virtual Draw
  Member of:     public View
  Defined in:    View.cpp
  Arguments:     BRect a_update_rect - the rect to be updated.
  Returns:       none
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual void Draw(BRect a_update_rect);
		
/*--------------------------------------------------------------------*\
  Function name: inline Status const
  Member of:     public View
  Arguments:     none
  Returns:       status_t - B_NO_ERROR if the object is valid, another
                     error code if not (see below).
  Throws:        none
  Description: Function to check if the object is valid. This function
      must be called after creating the object to insure that its
      instantiation was successful. Possible return values are as
      follows:
      
      B_NO_ERROR - the object was created successfully.
\*--------------------------------------------------------------------*/
		
		inline status_t Status(void) const
		{
			return m_status;
		}
	
	private:
	
		typedef BView Inherited;
		
		// Private data members
		status_t m_status;
};


#endif    // VIEW_H
