/*--------------------------------------------------------------------*\
  File:      Arrayomatic.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1996, 1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file defining the Arrayomatic template class.
  ID:        $Id: Arrayomatic.h,v 1.46 1998/07/21 19:55:39 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
  --------------------------------------------------------------------
  // Example file

  #include "Arrayomatic.h"
  #include <stdio.h>
  #include <string.h>
  
  typedef const char *CnstStr;
  
  const unsigned short k_num_months(12);
  const unsigned short k_num_days(7);
  
  const CnstStr g_months[k_num_months]
  {
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December"
  };
  
  const CnstStr g_days[k_num_days]
  {
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday"
  };
  
  // Simple function that acts as a wrapper for strcmp to match the
  // function pointer required by Arrayomatic.Member()
  inline int stringCompare(const CnstStr * const a_1,
    const CnstStr * const a_2)
  {
    return strcmp(*a_1, *a_2);
  }
  
  void main(void)
  {
    int err;
    size_t i, num_popped;
    CnstStr cur_item;
    
    Arrayomatic<CnstStr> array(10, &err);    // Create a new array
                                             // with an initial
                                             // capacity of 10
    
    // Check for an instantiation error
    if (err != ENOERR)
    {
      fprintf(stderr, "Couldn't create \"array\"!\n");
      return;
    }
    
    // Use array as a queue
    printf("### Using array as a queue ###\n");
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    
    // Look at g_months
    printf("Looking at \"g_months\"...\n");
    
    for (i = 0; i < k_num_months; i++)
    {
      printf("\tg_months[%d] = %s\n", i, g_months[i]);
    }
    
    // Insert the values into array
    printf("Pushing g_months onto \"array\"...\n");
    array.Push(g_months, k_num_months);
    
    // Make copy of the array
    printf("Copying \"array\" into \"new_array\"...\n");
    Arrayomatic<CnstStr> new_array = array;
    
    // Check for an instantiation error
    if (new_array.Status() != ENOERR)
    {
      fprintf(stderr, "Couldn't create \"new_array\"!\n");
      return;
    }
    
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    printf("There are currently %d items in \"new_array\".\n",
      new_array.NumItems());
    
    // Remove the values from array
    printf("Popping \"array\" onto g_months...\n");
    num_popped = array.Pop(g_months, array.NumItems());
    
    // Look at g_months now
    printf("Looking at \"g_months\"...\n");
    
    for (i = 0; i < num_popped; i++)
    {
      printf("\tg_months[%d] = %s\n", i, g_months[i]);
    }
    
    // Use array as a stack
    printf("### Using array as a stack ###\n");
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    printf("There are currently %d items in \"new_array\".\n",
      new_array.NumItems());
    
    // Push the values onto array
    printf("Pushing items onto \"array\"...\n");
    
    for (i = 0; i < k_num_days; i++)
    {
      printf("\tPushing %s...\n", g_days[i]);
      array.PushItem(g_days[i]);
    }
    
    printf("Done.\n");
    
    // Removing the values from new_array
    printf("Popping items off of \"new_array\"...\n");
    
    while (new_array.NumItems() > 0)
    {
      if (new_array.PopItem(cur_item) == ENOERR)
      {
        printf("\tPopped %s...\n", cur_item);
      }
    }
    
    printf("Done.\n");
    
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    printf("There are currently %d items in \"new_array\".\n",
      new_array.NumItems());
    
    // Looking at the values in array
    printf("Looking for \"%s\" in \"array\"...\n", g_months[0]);
    
    if (array.Member(g_months[0], stringCompare))
    {
      printf("\"%s\" found in \"array\".\n", g_months[0]);
    }
    else
    {
      printf("\"%s\" not found in \"array\".\n", g_months[0]);
    }
    
    for (i = 0; i < k_num_days; i++)
    {
      printf("\tarray[%d] = %s", i, array[i]);
      
      if (i == 2 % k_num_days)
      {
        printf(" -- (changing to \"%s\")", g_months[0]);
        array[i] = g_months[0];
      }
      
      printf("\n");
    }
    
    // Looking at the values in array
    printf("Looking for \"%s\" in \"array\"...\n", g_months[0]);
    
    if (array.Member(g_months[0], stringCompare))
    {
      printf("\"%s\" found in \"array\".\n", g_months[0]);
    }
    else
    {
      printf("\"%s\" not found in \"array\".\n", g_months[0]);
    }
    
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    
    // Use array to pop and push multiple items
    printf("### Popping and pushing multiple items ###\n");
    
    // Push all of g_months onto array
    printf("Pushing g_months onto \"array\"...\n");
    array.Push(g_months, k_num_months);
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    
    // Look at array
    printf("Looking at \"array\"...\n");
    
    for (i = 0; i < array.NumItems(); i++)
    {
      printf("\tarray[%d] = %s\n", i, array[i]);
    }
    
    // Pop some of array onto new_array
    printf("Popping some of \"array\" onto \"new_array\"...\n");
    array.Pop(new_array, k_num_days);
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    printf("There are currently %d items in \"new_array\".\n",
      new_array.NumItems());
    
    // Look at new_array
    printf("Looking at \"new_array\"...\n");
    
    for (i = 0; i < new_array.NumItems(); i++)
    {
      printf("\tnew_array[%d] = %s\n", i, new_array[i]);
    }
    
    // Pop all of new_array onto array
    printf("Popping \"new_array\" onto \"array\"...\n");
    new_array.Pop(array, new_array.NumItems(), false);
    printf("There are currently %d items in \"array\".\n",
      array.NumItems());
    printf("There are currently %d items in \"new_array\".\n",
      new_array.NumItems());
    
    // Look at array
    printf("Looking at \"array\"...\n");
    
    for (i = 0; i < array.NumItems(); i++)
    {
      printf("\tarray[%d] = %s\n", i, array[i]);
    }
    
    // Sort array
    printf("Sorting \"array\"...\n");
    array.Sort(stringCompare);
    
    // Look at array
    printf("Looking at \"array\"...\n");
    
    for (i = 0; i < array.NumItems(); i++)
    {
      printf("\tarray[%d] = %s\n", i, array[i]);
    }
    
    exit(0);
  }
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <errno.h>
#include <stdlib.h>


#ifndef ARRAYOMATIC_H
#define ARRAYOMATIC_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
  Class name:       template <class Type> Arrayomatic
  Inherits from:    none
  New data members: private int m_status - the status of the object.
                    private size_t m_capacity - contains the current
                        capacity of the array.
                    private size_t m_head - contains index of the
                        array of the item at the head of the array.
                        Unless there is no data (i.e., NumItems() ==
                        0), this always points to an index with valid
                        data in it.
                    private size_t m_tail - contains index of the
                        array of the item at the tail of the array.
                        This always points to an index without valid
                        data in it.
                    private Type *m_array - pointer to the array.
  Description: Class to implement a variable-sized generic array.
\*--------------------------------------------------------------------*/

template <class Type> class Arrayomatic
{
	private:
	
		// Private static member functions
		
/*--------------------------------------------------------------------*\
  Function name: static DefaultEqualityFunc
  Member of:     private template <class Type> Arrayomatic<Type>
  Arguments:     const Type * const a_1 - the first item to compare.
                 const Type * const a_2 - the first item to compare.
  Returns:       int - zero if the two items are equal, nonzero if
                     not.
  Throws:        none
  Description: Function to use the == operator to compare two items.
\*--------------------------------------------------------------------*/
		
		static int DefaultEqualityFunc(const Type * const a_1, const Type * const a_2)
		{
			return (*a_1 != *a_2);
		}
	
	public:
	
		// Public data members
		typedef int (*CompFunc)(const Type * const, const Type * const);
		
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: Arrayomatic
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const size_t a_capacity - the capacity of the array
                     to be created. Default: 64.
                 int * const a_err - a placeholder for the error
                     encountered during construction (see below).
                     Default: NULL.
  Throws:        none
  Description: Default class constructor. You must check the value of
      a_err or the return value of Status() after invoking the
      constructor to see if the object is usable. See the Status()
      function (below) for possible error values.
\*--------------------------------------------------------------------*/
		
		Arrayomatic(const size_t a_capacity = 64, int * const a_err = NULL) :
			m_status(ENOERR),
			m_capacity(a_capacity),
			m_head(0),
			m_tail(0),
			m_array((m_capacity == 0) ? NULL : new Type[m_capacity])
		{
			// Check for an allocation error
			if (m_capacity != 0
				&& m_array == NULL)
			{
				m_status = ENOMEM;
				m_capacity = 0;
			}
			
			// Give the error back to the caller
			if (a_err != NULL)
			{
				*a_err = Status();
			}
		}
		
/*--------------------------------------------------------------------*\
  Function name: Arrayomatic
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const Type a_items[] - an array of items to be
                     pushed onto the newly-created array.
                 const size_t a_num - the number of items to push.
                     Default: 1.
                 int * const a_err - a placeholder for the error
                     encountered during construction (see below).
                     Default: NULL.
  Throws:        none
  Description: Class constructor. You must check the value of a_err
      or the return value of Status() after invoking the constructor
      to see if the object is usable. See the Status() function
      (below) for possible error values.
\*--------------------------------------------------------------------*/
		
		Arrayomatic(const Type a_items[], const size_t a_num = 1, int * const a_err = NULL) :
			m_status(ENOERR),
			m_capacity(a_num),
			m_head(0),
			m_tail(0),
			m_array((m_capacity == 0) ? NULL : new Type[m_capacity])
		{
			// Check for an allocation error
			if (m_capacity != 0
				&& m_array == NULL)
			{
				m_status = ENOMEM;
				m_capacity = 0;
			}
			else
			{
				m_status = Push(a_items, a_num);
			}
			
			// Give the error back to the caller
			if (a_err != NULL)
			{
				*a_err = Status();
			}
		}
		
/*--------------------------------------------------------------------*\
  Function name: Arrayomatic
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const Arrayomatic<Type> &a_obj - the object to copy.
  Throws:        none
  Description: Class copy constructor. You must use the Status()
      function (see below) after invoking the constructor to see if
      the object is usable.
\*--------------------------------------------------------------------*/
		
		Arrayomatic(const Arrayomatic<Type> &a_obj) :
			m_status(ENOERR),
			m_capacity(0),
			m_head(0),
			m_tail(0),
			m_array(NULL)
		{
			operator=(a_obj);
		}
		
/*--------------------------------------------------------------------*\
  Function name: ~Arrayomatic
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		~Arrayomatic(void)
		{
			delete [] m_array;
		}
		
/*--------------------------------------------------------------------*\
  Function name: operator=
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const Arrayomatic &a_obj - the object to copy.
  Returns:       Arrayomatic & - the assigned object.
  Throws:        none
  Description: Function to overload the = operator. You must use the
      Status() function (see below) after invoking this function to
      see if the object is usable.
\*--------------------------------------------------------------------*/
		
		Arrayomatic &operator=(const Arrayomatic &a_obj)
		{
			if (&a_obj == this)
			{
				return *this;
			}
			
			// Assign the new values
			m_status = a_obj.m_status;
			m_head = a_obj.m_head;
			m_tail = a_obj.m_tail;
			
			// If it's the same size, we don't need to do
			// anything
			if (m_capacity != a_obj.m_capacity)
			{
				m_capacity = a_obj.m_capacity;
				
				// Get rid of the old array
				delete [] m_array;
				
				// Check for an allocation error
				if ((m_array = new Type[m_capacity]) == NULL)
				{
					m_status = ENOMEM;
					m_capacity = 0;
					return *this;
				}
			}
			
			// Copy the array
			for (size_t i(m_head); i < m_tail; i++)
			{
				m_array[i] = a_obj.m_array[i];
			}
			
			return *this;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline operator[]
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const size_t a_ind - the index of the array.
  Returns:       Type & - reference to the data associated with the
                     given index.
  Throws:        none
  Description: Function to overload the [] operator to allow access to
      the item of a given index in the array without removing it.
\*--------------------------------------------------------------------*/
		
		inline Type &operator[](const size_t a_ind)
		{
			return m_array[m_head + a_ind];
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline operator <Type *>
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Returns:       Type * - the array.
  Throws:        none
  Description: Function to overload the <Type *> conversion operator
      for the array.
\*--------------------------------------------------------------------*/
		
		inline operator Type*(void)
		{
			return &m_array[m_head];
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Capacity const
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Returns:       size_t - the capacity of the array.
  Throws:        none
  Description: Function to return the capacity of the array.
\*--------------------------------------------------------------------*/
		
		inline size_t Capacity(void) const
		{
			return m_capacity;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Clear
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Function to reset the array.
\*--------------------------------------------------------------------*/
		
		inline void Clear(void)
		{
			m_head = 0;
			m_tail = 0;
			m_status = ENOERR;
		}
		
/*--------------------------------------------------------------------*\
  Function name: Insert
  Member of:     private template <class Type> Arrayomatic<Type>
  Arguments:     Type &a_item - the item to be inserted.
                 const size_t a_ind - the index at which to insert the
                     item.
  Returns:       int - the error code associated with the operation
                     (see below).
  Throws:        none
  Description: Function to insert a given item into the array at a
      given array. Possible return values are as follows:
      
      ENOERR - success.
      ENOMEM - there was not enough memory to perform the insertion.
      ERANGE - the given index was out of range.
\*--------------------------------------------------------------------*/
		
		int Insert(const Type &a_item, const size_t a_ind)
		{
			if (a_ind > NumItems())
			{
				return ERANGE;
			}
			
			// Increase the table capacity if necessary
			if (m_capacity <= NumItems()
				&& Resize((m_capacity < 64) ? 64 : 2 * m_capacity) != ENOERR)
			{
				return ENOMEM;
			}
			
			// Make sure there's room on the end
			if (m_tail >= m_capacity
				|| (m_head > 0
					&& a_ind < NumItems() / 2))
			{
				// Push everything downwards
				for (size_t i(--m_head); i < m_head + a_ind; i++)
				{
					m_array[i] = m_array[i + 1];
				}
			}
			else
			{
				// Push everything upwards
				for (size_t i(m_tail++); i > m_head + a_ind; i--)
				{
					m_array[i] = m_array[i - 1];
				}
			}
			
			(*this)[a_ind] = a_item;
			
			return ENOERR;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Member
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const Type &a_item - the item for which to search.
                 CompFunc a_comp_func - the comparison function.
                     Default: DefaultEqualityFunc.
  Returns:       bool - true if a_item is in the array, false if not.
  Throws:        none
  Description: Function to find if a given item is in the array using
      a given comparison function. The only requirements on the
      comparison function is that it should return zero if two
      compared items are equivalent, and nonzero if not. If the
      comparison function is NULL, then the == operator is used for
      comparisons.
\*--------------------------------------------------------------------*/
		
		inline bool Member(const Type &a_item, CompFunc a_comp_func = DefaultEqualityFunc)
		{
			if (NumItems() == 0)
			{
				return false;
			}
			
			size_t i;
			
			for (i = 0; i < NumItems()
				&& (*a_comp_func)(&a_item, &(*this)[i]) != 0; i++)
			{
				;
			}
			
			return (i < NumItems());
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline NumItems const
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Returns:       size_t - the number of items in the array.
  Throws:        none
  Description: Function to return the number of items in the array.
\*--------------------------------------------------------------------*/
		
		inline size_t NumItems(void) const
		{
			return m_tail - m_head;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Pop
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Type a_items[] - an array into which the items to be
                     popped are placed. This array should be able to
                     hold up to a_num items.
                 const size_t a_num - the number of items to pop.
                     Default: 1.
                 const bool a_from_head - true if the end from which
                     to pop the items is the head, false if it's the
                     tail. Default: true.
  Returns:       size_t - the number of items actually popped. If this
                     is less than the desired number, then either
                     there were not enough items to be popped (which
                     can be verified by checking that NumItems() == 0)
                     or an error occured.
  Throws:        none
  Description: Function to pop a given number of items from the array
      into another array. This function returns the exact number of
      items popped.
\*--------------------------------------------------------------------*/
		
		inline size_t Pop(Type a_items[], const size_t a_num = 1, const bool a_from_head = true)
		{
			size_t i;
			
			// Copy them into the array
			for (i = 0; i < a_num
				&& PopItem(a_items[i], a_from_head) == ENOERR; i++)
			{
				;
			}
			
			return i;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Pop
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Arrayomatic<Type> &a_items - an array into which the
                     items to be popped are placed.
                 const size_t a_num - the number of items to pop.
                     Default: 1.
                 const bool a_from_head - true if the end of this
                     array from which to pop the items is the head,
                     false if it's the tail. Default: true.
                 const bool a_to_tail - true if the end on which to
                     push the items from this array is the tail, false
                     if it's the head. Default: true.
  Returns:       size_t - the number of items actually popped. If this
                     is less than the desired number, then either
                     there were not enough items to be popped (which
                     can be verified by checking that NumItems() == 0)
                     or an error occured.
  Throws:        none
  Description: Function to pop a given number of items from the array
      into another array. This function will attempt to push them onto
      the other array, rather than replacing it with the items popped.
      This function returns the exact number of items popped.
\*--------------------------------------------------------------------*/
		
		inline size_t Pop(Arrayomatic<Type> &a_items, const size_t a_num = 1, const bool a_from_head = true, const bool a_to_tail = true)
		{
			int result;
			size_t i;
			Type item;
			
			for (i = 0; i < a_num
				&& PopItem(item, a_from_head) == ENOERR; i++)
			{
				if ((result = a_items.PushItem(item, a_to_tail)) != ENOERR)
				{
					break;
				}
			}
			
			return i;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline PopItem
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Type &a_item - the item to be popped.
                 const bool a_from_head - true if the end of this
                     array from which to pop the item is the head,
                     false if it's the tail. Default: true.
  Returns:       int - the error code associated with the operation
                     (see below).
  Throws:        none
  Description: Function to pop an item from the array. Possible return
      values are as follows:
      
      ENOERR - success.
      ERANGE - there's nothing to pop.
\*--------------------------------------------------------------------*/
		
		inline int PopItem(Type &a_item, const bool a_from_head = true)
		{
			return Remove(a_item, (a_from_head) ? 0 :  NumItems() - 1);
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Push
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     const Type a_items[] - an array of items to be
                     pushed.
                 const size_t a_num - the number of items to push.
                     Default: 1.
                 const bool a_to_tail - true if the end of this array
                     on which to push the items is the tail, false if
                     it's the head. Default: true.
  Returns:       size_t - the number of items actually pushed. If this
                     is less than the desired number, an error
                     occured.
  Throws:        none
  Description: Function to push a given number of items onto the
      array.
\*--------------------------------------------------------------------*/
		
		inline size_t Push(const Type a_items[], const size_t a_num = 1, const bool a_to_tail = true)
		{
			size_t i;
			
			// If we're pushing onto the tail, read first
			// from the head
			if (a_to_tail)
			{
				for (i = 0; i < a_num; i++)
				{
					if (PushItem(a_items[i], a_to_tail) != ENOERR)
					{
						break;
					}
				}
			}
			// If we're pushing onto the head, read first
			// from the tail
			else
			{
				for (i = a_num; i > 0; i--)
				{
					if (PushItem(a_items[i - 1], a_to_tail) != ENOERR)
					{
						break;
					}
				}
			}
			
			return i;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Push
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Arrayomatic<Type> &a_items - an array from which the
                     items are to be popped.
                 const size_t a_num - the number of items to push.
                     Default: 1.
                 const bool a_to_tail - true if the end of this array
                     on which to push the items is the tail, false if
                     it's the head. Default: true.
                 const bool a_from_head - true if the end from which
                     to pop the items for this array is the head,
                     false if it's the tail. Default: true.
  Returns:       size_t - the number of items actually popped. If this
                     is less than the desired number, then either
                     there were not enough items to be popped (which
                     can be verified by checking that NumItems() == 0)
                     or an error occured.
  Throws:        none
  Description: Function to pop a given number of items from a given
      array onto the array.
\*--------------------------------------------------------------------*/
		
		inline size_t Push(Arrayomatic<Type> &a_items, const size_t a_num = 1, const bool a_to_tail = true, const bool a_from_head = true)
		{
			size_t i;
			Type item;
			
			// If we're pushing onto the tail, pop first
			// from the head, otherwise pop first from the
			// tail
			for (i = 0; i < a_num
				&& a_items.PopItem(item, a_from_head) == ENOERR; i++)
			{
				if (PushItem(item, a_to_tail) != ENOERR)
				{
					break;
				}
			}
			
			return i;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline PushItem
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Type &a_item - the item to be pushed.
                 const bool a_to_tail - true if the end of this array
                     on which to push the item is the tail, false if
                     it's the head. Default: true.
  Returns:       int - the error code associated with the operation
                     (see below).
  Throws:        none
  Description: Function to push an item onto the array. Possible
      return values are as follows:
      
      ENOERR - success.
      ENOMEM - there was not enough memory to perform the insertion.
\*--------------------------------------------------------------------*/
		
		inline int PushItem(const Type &a_item, const bool a_to_tail = true)
		{
			return Insert(a_item, (a_to_tail) ? NumItems() : 0);
		}
		
/*--------------------------------------------------------------------*\
  Function name: Remove
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Type &a_item - the item removed.
                 const size_t a_ind - the index from which to remove
                     the item.
  Returns:       int - the error code associated with the operation
                     (see below).
  Throws:        none
  Description: Function to remove a given item at a given index from
      the array. Possible return values are as follows:
      
      ENOERR - success.
      ERANGE - the given index was out of range.
\*--------------------------------------------------------------------*/
		
		int Remove(Type &a_item, const size_t a_ind)
		{
			if (a_ind >= NumItems())
			{
				return ERANGE;
			}
			
			// Get the item
			a_item = (*this)[a_ind];
			
			// Remove the item from the list
			if (a_ind >= NumItems() / 2)
			{
				// Move everything down
				m_tail--;
				
				for (size_t i(m_head + a_ind); i < m_tail; i++)
				{
					m_array[i] = m_array[i + 1];
				}
			}
			else
			{
				// Move everything up
				m_head++;
				
				for (size_t i(m_head + a_ind); i > m_head; i--)
				{
					m_array[i] = m_array[i - 1];
				}
			}
			
			return ENOERR;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Remove
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     Type &a_item - the item to be removed.
                 CompFunc a_comp_func - the comparison function.
                     Default: DefaultEqualityFunc.
  Returns:       bool - false if the item could not be removed (i.e.,
                     it could not be found), true otherwise.
  Throws:        none
  Description: Function to remove a given item from the array using a
      given comparison function. The only requirements on the
      comparison function is that it should return zero if two
      compared items are equivalent, and nonzero if not. If the
      comparison function is NULL, then the == operator is used for
      comparisons.
\*--------------------------------------------------------------------*/
		
		inline bool Remove(const Type &a_item, CompFunc a_comp_func = DefaultEqualityFunc)
		{
			size_t i;
			
			// Look through the list
			for (i = 0; i < NumItems(); i++)
			{
				// If we've found it, remove it
				if ((*a_comp_func)(&a_item, &(*this)[i]) == 0)
				{
					Type item;
					return (Remove(item, i) == ENOERR);
				}
			}
			
			return false;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Sort
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     CompFunc a_comp_func - the comparison function.
  Returns:       none
  Throws:        none
  Description: Function to sort the array using a given comparison
      function. The only requirements on the comparison function is
      that it should return zero if two compared items are equivalent,
      less than zero if the first item is "less than" the second and
      greater than zero if the first item is "greater than" the second
      (e.g., strcmp()). This function uses the QuickSort algorithm.
\*--------------------------------------------------------------------*/
		
		inline void Sort(CompFunc a_comp_func)
		{
			qsort(static_cast<Type *>(*this), NumItems(), sizeof (Type), reinterpret_cast<_compare_function>(a_comp_func));
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline Status const
  Member of:     public template <class Type> Arrayomatic<Type>
  Arguments:     none
  Returns:       int - ENOERR if the object is valid, another error
                     code if not (see below).
  Throws:        none
  Description: Function to check if the object is valid. This function
      must be called after creating the object to insure that its
      instantiation was successful. Possible return values are as
      follows:
      
      ENOERR - the object was created successfully.
      ENOMEM - on construction, there was not enough memory available
          to create the object.
\*--------------------------------------------------------------------*/
		
		inline int Status(void) const
		{
			return m_status;
		}
	
	private:
	
		// Private data members
		int m_status;
		size_t m_capacity;
		size_t m_head;
		size_t m_tail;
		Type *m_array;
		
		// Private member functions
		
/*--------------------------------------------------------------------*\
  Function name: Resize
  Member of:     private template <class Type> Arrayomatic<Type>
  Arguments:     const size_t a_capacity - the new capacity.
  Returns:       int - the error code associated with the operation
                     (see below).
  Throws:        none
  Description: Function to resize the array. Possible return values
      are as follows:
      
      ENOERR - success.
      ENOMEM - there was not enough memory available to resize the
          array.
      ERANGE - the given capacity was too small.
\*--------------------------------------------------------------------*/
		
		int Resize(const size_t a_capacity)
		{
			// If it's the same capacity, we don't need to
			// do anything
			if (a_capacity == m_capacity)
			{
				return ENOERR;
			}
			
			// Check the new capacity
			if (a_capacity < NumItems())
			{
				return ERANGE;
			}
			
			// Save the old values
			size_t old_capacity(m_capacity);
			Type *old_array(m_array);
			m_capacity = a_capacity;
			
			// Check for an allocation error
			if ((m_array = new Type[m_capacity]) == NULL)
			{
				m_capacity = old_capacity;
				m_array = old_array;
				return ENOMEM;
			}
			
			// Copy the data into the new array
			for (size_t i(0); i < NumItems(); i++)
			{
				m_array[i] = old_array[m_head + i];
			}
			
			// Set the new head and tail
			m_tail -= m_head;
			m_head = 0;
			
			// Clean up
			delete [] old_array;
			
			return ENOERR;
		}
};


#endif    // ARRAYOMATIC_H
